//+--------------------------------------------------------------------------------------+
//|                                                                                      |
//+--------------------------------------------------------------------------------------+
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1 DeepSkyBlue
#property indicator_color2 DimGray
#property indicator_color3 PaleVioletRed
#property indicator_width1 2
#property indicator_width3 2
#property indicator_style2 STYLE_DOT

#import "libSSA.dll"
   void fastSingular(double& sourceArray[],int arraySize, int lag, int numberOfComputationLoops, double& destinationArray[]);
#import "fourier extrapolation.dll"
   void fourierExtrapolation(double& source[], double& past[], double& prediction[], int pastBars, int lastBar, int futureBars, int harmonicsNo, double harmonicsTolerance);
#import

//
//
//
//
//

extern int     SSALag                  = 25;
extern int     SSANumberOfComputations =  2;
extern int     SSAPeriodNormalization  = 10;
extern int     SSANumberOfBars         = 2000;
extern string  _                       = "Fourier extrapolation parameters";
extern int     LastBar                 = 0;      // Last bar in the past data
extern int     PastBars                = 500;    // Number of past bars
extern int     FutBars                 = 50;     // Number of bars to predict 
extern int     HarmNo                  = 50;     // Number of frequencies; HarmNo=0 computes PastBars harmonics
extern double  FreqTOL                 = 0.0001; // Tolerance of frequency calculation - FreqTOL > 0.001 may not converge

//
//
//
//
//

double in[];
double pv[];
double fv[];
double norm[];

double source[];
double result[];
double prediction[];
double ssaIn[];
double ssaOut[];

//+--------------------------------------------------------------------------------------+
//|                                                                                      |
//+--------------------------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(4);
   if (HarmNo==0 || HarmNo>PastBars) HarmNo=PastBars;
            SetIndexBuffer(0,in); SetIndexLabel(0,"values");
            SetIndexBuffer(1,pv); SetIndexLabel(1,"past");       SetIndexShift(1,       -LastBar);
            SetIndexBuffer(2,fv); SetIndexLabel(2,"prediction"); SetIndexShift(2,FutBars-LastBar);
            SetIndexBuffer(3,norm);
   IndicatorShortName("Fourier extrapolation of full SSA normalized");
   return(0);
}
int deinit(){return(0);}

//+--------------------------------------------------------------------------------------+
//|                                                                                      |
//+--------------------------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int limit,counted_bars = IndicatorCounted();

      if(counted_bars < 0) return(-1);
      if(counted_bars > 0) counted_bars--;
         limit = MathMin(Bars-counted_bars+SSALag,Bars-1);

         int np      = PastBars;        if (np>Bars)    np      = Bars;
         int nf      = FutBars;         if (nf<0)       nf      = 0;
         int ssaBars = SSANumberOfBars; if (ssaBars<np+nf) ssaBars = MathMin(5000,np+nf);

         if (ArraySize(source)     != PastBars)  ArrayResize(source,PastBars);
         if (ArraySize(result)     != PastBars)  ArrayResize(result,PastBars);
         if (ArraySize(prediction) != FutBars+1) ArrayResize(prediction,FutBars+1);
         if (ArraySize(ssaIn)      != ssaBars)
         {
            ArrayResize(ssaIn ,ssaBars);
            ArrayResize(ssaOut,ssaBars);
         }

      //
      //
      //
      //
      //

      for(int i=limit; i>=0; i--)
      {
         double ma  = iMA(NULL,0,SSAPeriodNormalization,0,MODE_SMA,PRICE_CLOSE,i);
         double dev = iStdDev(NULL,0,SSAPeriodNormalization,0,MODE_SMA,PRICE_CLOSE,i)*3.0;
      
         norm[i] = (Close[i]-ma)/(MathMax(dev,0.000001));
      }

      ArrayCopy(ssaIn,norm,0,0,ssaBars);
               fastSingular(ssaIn,ssaBars,SSALag,SSANumberOfComputations,ssaOut);
      ArrayCopy(in,ssaOut);

      ArrayCopy(source,in,0,LastBar,np);
               fourierExtrapolation(source,result,prediction,np,LastBar,nf,HarmNo,FreqTOL);
      ArrayCopy(pv,result);
      ArrayCopy(fv,prediction);

      //
      //
      //
      //
      //
      
      SetIndexDrawBegin(1,Bars-np);
      SetIndexDrawBegin(2,Bars-nf-1);
   return(0); 
}